@@ -6,8 +6,6 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
-from __future__ import absolute_import
-
 import errno
 import logging
 import os
@@ -50,7 +48,7 @@ def open(url, mode="r", sparse=False, dirty=False, max_connections=8,
         raise
 
 
-class Backend(object):
+class Backend:
     """
     Base class for file backends.
     """
