@@ -6,8 +6,6 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
-from __future__ import absolute_import
-
 import argparse
 import configparser
 import glob
@@ -18,6 +16,7 @@ import logging.config
 import os
 import pwd
 import signal
+import socket
 import sys
 
 import systemd.daemon
@@ -41,7 +40,8 @@ def main():
             return
 
         configure_logger(cfg)
-        log.info("Starting (pid=%s, version=%s)", os.getpid(), version.string)
+        log.info("Starting (hostname=%s pid=%s, version=%s)",
+                 socket.gethostname(), os.getpid(), version.string)
 
         server = Server(cfg)
         signal.signal(signal.SIGINT, server.terminate)
