@@ -6,9 +6,6 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
-from __future__ import absolute_import
-from __future__ import division
-
 import sys
 import threading
 import time
@@ -16,7 +13,7 @@ import time
 from .. _internal import util
 
 
-class ProgressBar(object):
+class ProgressBar:
 
     def __init__(self, size=0, output=sys.stdout, step=0.1, width=79,
                  now=time.monotonic):
